@@ -3,8 +3,9 @@ from datetime import datetime
 from pandas import Categorical, Series
 from scipy.sparse import lil_matrix
 from sklearn.base import BaseEstimator, TransformerMixin
+from sklearn.pipeline import Pipeline
 from sklearn.utils import column_or_1d
-from sklearn2pmml.util import cast, eval_rows, flat_transform
+from sklearn2pmml.util import cast, eval_rows, dt_transform
 
 import numpy
 import pandas
@@ -24,6 +25,14 @@ def _col2d(X):
 		X = X.values
 	return X.reshape(-1, 1)
 
+def _int(X):
+	if numpy.isscalar(X):
+		return int(X)
+	else:
+		if isinstance(X, Series):
+			X = X.values
+		return X.astype(int)
+
 class Aggregator(BaseEstimator, TransformerMixin):
 
 	def __init__(self, function):
@@ -98,9 +107,10 @@ class DurationTransformer(BaseEstimator, TransformerMixin):
 		return self
 
 	def transform(self, X):
-		def to_duration(X):
-			return self._to_duration(pandas.to_timedelta(X - self.epoch))
-		return flat_transform(X, to_duration)
+		def to_int_duration(X):
+			duration = self._to_duration(pandas.to_timedelta(X - self.epoch))
+			return _int(duration)
+		return dt_transform(X, to_int_duration)
 
 class DaysSinceYearTransformer(DurationTransformer):
 
@@ -108,7 +118,7 @@ class DaysSinceYearTransformer(DurationTransformer):
 		super(DaysSinceYearTransformer, self).__init__(year)
 
 	def _to_duration(self, td):
-		return ((td.days).values).astype(int)
+		return td.days
 
 class SecondsSinceYearTransformer(DurationTransformer):
 
@@ -116,7 +126,7 @@ class SecondsSinceYearTransformer(DurationTransformer):
 		super(SecondsSinceYearTransformer, self).__init__(year)
 
 	def _to_duration(self, td):
-		return ((td.total_seconds()).values).astype(int)
+		return td.total_seconds()
 
 class SecondsSinceMidnightTransformer(BaseEstimator, TransformerMixin):
 
@@ -124,16 +134,17 @@ class SecondsSinceMidnightTransformer(BaseEstimator, TransformerMixin):
 		pass
 
 	def _to_duration(self, td):
-		return ((td.seconds).values).astype(int)
+		return td.seconds
 
 	def fit(self, X, y = None):
 		return self
 
 	def transform(self, X):
-		def to_duration(X):
+		def to_int_duration(X):
 			dt = pandas.to_datetime(X)
-			return self._to_duration(dt - dt.normalize())
-		return flat_transform(X, to_duration)
+			duration = self._to_duration(dt - dt.normalize())
+			return _int(duration)
+		return dt_transform(X, to_int_duration)
 
 class ExpressionTransformer(BaseEstimator, TransformerMixin):
 
@@ -341,6 +352,25 @@ class SubstringTransformer(BaseEstimator, TransformerMixin):
 		Xt = eval_rows(X, func)
 		return _col2d(Xt)
 
+class WordCountTransformer(BaseEstimator, TransformerMixin):
+
+	def __init__(self, word_pattern = "\w+", non_word_pattern = "\W+"):
+		self.word_pattern = word_pattern
+		self.non_word_pattern = non_word_pattern
+		self.pipeline_ = Pipeline([
+			("word_replacer", ReplaceTransformer(pattern = "({0})".format(word_pattern), replacement = "1")),
+			("non_word_replacer", ReplaceTransformer(pattern = "({0})".format(non_word_pattern), replacement = "")),
+			("counter", ExpressionTransformer("len(X[0])", dtype = int))
+		])
+
+	def fit(self, X, y = None):
+		X = column_or_1d(X, warn = True)
+		return self
+
+	def transform(self, X):
+		X = column_or_1d(X, warn = True)
+		return self.pipeline_.transform(X)
+
 class StringNormalizer(BaseEstimator, TransformerMixin):
 
 	def __init__(self, function = None, trim_blanks = True):
